#include "IniSyntax.hxx"

// --- internal constants ---

const QString keywords("set");

// --- constructors and deconstructors ---

IniSyntax::IniSyntax(QObject *parent)
: QSyntaxHighlighter(parent)
{
    setup();
}

IniSyntax::IniSyntax(QTextDocument *parent)
: QSyntaxHighlighter(parent)
{
    setup();
}

IniSyntax::~IniSyntax()
{
}

// --- protected methods ---

void IniSyntax::highlightBlock(const QString& text)
{
    qint32 index = 0;

    setCurrentBlockState(0);

    // keywords
    index = text.indexOf(_keywords_re);
    while (index >= 0)
    {
        auto len = _keywords_re.matchedLength();

        setFormat(index, len, _keywords_f);
        index = text.indexOf(_keywords_re, index + len);
    }

    // ints
    index = text.indexOf(_ints_re);
    while (index >= 0)
    {
        auto len = _ints_re.matchedLength();

        setFormat(index, len, _ints_f);
        index = text.indexOf(_ints_re, index + len);
    }

    // strings
    index = text.indexOf(_strings_re);
    while (index >= 0)
    {
        auto len = _strings_re.matchedLength();

        setFormat(index, len, _strings_f);
        index = text.indexOf(_strings_re, index + len);
    }

    // single line comments
    index = text.indexOf(_comment_re);
    if (index >= 0)
        setFormat(index, _comment_re.matchedLength(), _comment_f);
}

void IniSyntax::setup()
{
    QString pattern;

    // format
    _keywords_f.setFontWeight(QFont::Bold);
    _keywords_f.setForeground(Qt::green);

    _ints_f.setFontWeight(QFont::Bold);
    _ints_f.setForeground(Qt::darkBlue);

    _strings_f.setFontItalic(true);
    _strings_f.setForeground(Qt::darkYellow);

    _comment_f.setFontItalic(true);
    _comment_f.setForeground(Qt::darkGray);


    // regexp
    pattern.append("\\b(");
    pattern.append(keywords.split(" ").join("|"));
    pattern.append(")\\b");
    _keywords_re.setPattern(pattern);

    _ints_re.setPattern("\\b[0-9]+\\b");

    _strings_re.setPattern("(\"[^\"]*)\"");

    _comment_re.setPattern("//[^\n]*");
}
