#include <QMouseEvent>
#include <QVector3D>
#include "GLWidget.hxx"

// --- constructors and deconstructors ---

GLWidget::GLWidget(QWidget *parent, QGLWidget *shared)
: QGLWidget(parent, shared), _last_pos(0, 0), _rot_x(0.0), _rot_y(0.0), _rot_z(0.0)
{
}

GLWidget::~GLWidget()
{
}

// --- public slots ---

void GLWidget::slotRotateDelta(qreal angle_x, qreal angle_y, qreal angle_z)
{
    _rot_x += angle_x;
    _rot_y += angle_y;
    _rot_z += angle_z;
    updateGL();
}

// --- protected methods ---

void GLWidget::initializeGL()
{
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);;
}

void GLWidget::paintGL()
{
    qglClearColor(Qt::black);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glLoadIdentity();
    glTranslatef(0.0, 0.0, -10.0);
    glRotatef(_rot_x / 16.0, 1.0, 0.0, 0.0);
    glRotatef(_rot_y / 16.0, 0.0, 1.0, 0.0);
    glRotatef(_rot_z / 16.0, 0.0, 0.0, 1.0);

    if (_vertices.count())
    {
        glVertexPointer(3, GL_FLOAT, 0, _vertices.constData());
        glEnableClientState(GL_VERTEX_ARRAY);

        for (qint32 i = 0; i < _vertices.count(); ++i)
            glDrawArrays(GL_TRIANGLE_FAN, i * 4, 4);
    }
}

void GLWidget::resizeGL(qint32 width, qint32 height)
{
    const qint32 side = qMin(width, height);

    glViewport((width - side) / 2, (height - side) / 2, side, side);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-0.5, 0.5, 0.5, -0.5, 4.0, 15.0);
    glMatrixMode(GL_MODELVIEW);
}

void GLWidget::mousePressEvent(QMouseEvent *event)
{
    _last_pos = event->pos();
}

void GLWidget::mouseMoveEvent(QMouseEvent *event)
{
    qint32 dx = event->x() - _last_pos.x();
    qint32 dy = event->y() - _last_pos.y();

    if (event->buttons() & Qt::LeftButton)
        slotRotateDelta(8 * dy, 8 * dx, 0);
    else if (event->buttons() & Qt::RightButton)
        slotRotateDelta(8 * dy, 0, 8 * dx);
    else
        _last_pos = event->pos();
}

void GLWidget::mouseReleaseEvent(QMouseEvent * /*event*/)
{
    emit sigClicked();
}
