#include <QFile>
#include "filesystem/FileIdentifier.hxx"

// --- internal typedefs ---

using IdentifyType = NC::Identify::Type;

// --- constructors and deconstructors ---

FileIdentifier::FileIdentifier(QObject *parent)
: QObject(parent)
{
}

FileIdentifier::~FileIdentifier()
{
}

// --- public methodes ---

IdentifyType FileIdentifier::identify(const QString& filename) const
{
    QFile ifile(filename);
    quint32 data = 0;

    if (!ifile.exists())
        return IdentifyType::Error;

    if (ifile.open(QIODevice::ReadOnly))
    {
        ifile.read(reinterpret_cast<char *>(&data), sizeof (data));
        ifile.close();
    }
    else
        return IdentifyType::Error;

    return identifyData(data);
}

IdentifyType FileIdentifier::identify(const QByteArray& stream) const
{
    if (stream.size() < static_cast<const qint32>(sizeof (quint32)))
        return IdentifyType::Error;

    QByteArray data = stream.left(sizeof (quint32));

    return identifyData(data.toUInt());
}

QString FileIdentifier::identifyName(const IdentifyType& type) const
{
    switch (type)
    {
        case Error:
            return "file error";
        case Plain:
            return "common file";
        case ZLIB:
            return "ZLIB compressed file";
        case VFS:
            return "Neocron VFS file";
        case PAK1:
            return "Neocron PAK file";
        case PAK2:
            return "Neocron PAK file (2nd header)";
        case PAK3:
            return "Neocron PAK file (3rd header)";
        default:
            return "unknown file format";
    }
}

QString FileIdentifier::identifyName(const QString& filename) const
{
    return identifyName(identify(filename));
}

QString FileIdentifier::identifyName(const QByteArray& stream) const
{
    return identifyName(identify(stream));
}

// --- protected methods ---

IdentifyType FileIdentifier::identifyData(const quint32& data) const
{
    if (data == IdentifyType::VFS)
        return IdentifyType::VFS;

    if (data == IdentifyType::PAK1)
        return IdentifyType::PAK1;

    if (reinterpret_cast<const char *>(&data)[0] == 'x')
        return IdentifyType::ZLIB;

    return IdentifyType::Plain;
}
